@@ -1,4 +1,4 @@
-ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
+ChoosePalette <- function(pal=diverge_hcl, n=7L, parent=NULL) {
 # A GUI for selecting a color palette.
 
   # Additional functions (subroutines)
@@ -92,7 +92,7 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
   # Entry change
 
   EntryChange <- function(v, x.lim, x.ent.var, x.scl.var) {
-    x <- suppressWarnings(as.integer(tclvalue(x.ent.var)))
+    x <- suppressWarnings(as.numeric(tclvalue(x.ent.var)))
     if (is.na(x))
       return()
     if (x < x.lim[1]) {
@@ -109,31 +109,31 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
 
   # Get color palette as function of n
 
-  GetPalette <- function(h1, h2, c1, c2, l1, l2, p1, p2) {
+  GetPalette <- function(h1, h2, c1, c2, l1, l2, p1, p2, fixup=TRUE) {
     type <- as.character(tclvalue(nature.var))
     if (type == "Qualitative") {
       f <- rainbow_hcl
       formals(f) <- eval(substitute(alist(n=, c=d1, l=d2, start=d3, end=d4,
-                                          gamma=NULL, fixup=TRUE, ...=),
-                                    list(d1=c1, d2=l1, d3=h1, d4=h2)))
+                                          fixup=d5, gamma=NULL, ...=),
+                                    list(d1=c1, d2=l1, d3=h1, d4=h2, d5=fixup)))
     } else if (type == "Sequential (single hue)") {
       f <- sequential_hcl
       formals(f) <- eval(substitute(alist(n=, h=d1, c.=d2, l=d3, power=d4,
-                                          gamma=NULL, fixup=TRUE, ...=),
+                                          fixup=d5, gamma=NULL, ...=),
                                     list(d1=h1, d2=c(c1, c2), d3=c(l1, l2),
-                                         d4=p1)))
+                                         d4=p1, d5=fixup)))
     } else if (type == "Sequential (multiple hues)") {
       f <- heat_hcl
       formals(f) <- eval(substitute(alist(n=, h=d1, c.=d2, l=d3, power=d4,
-                                          gamma=NULL, fixup=TRUE, ...=),
+                                          fixup=d5, gamma=NULL, ...=),
                                     list(d1=c(h1, h2), d2=c(c1, c2),
-                                         d3=c(l1, l2), d4=c(p1, p2))))
+                                         d3=c(l1, l2), d4=c(p1, p2), d5=fixup)))
     } else if (type == "Diverging") {
       f <- diverge_hcl
       formals(f) <- eval(substitute(alist(n=, h=d1, c=d2, l=d3, power=d4,
-                                          gamma=NULL, fixup=TRUE, ...=),
+                                          fixup=d5, gamma=NULL, ...=),
                                     list(d1=c(h1, h2), d2=c1, d3=c(l1, l2),
-                                         d4=p1)))
+                                         d4=p1, d5=fixup)))
     }
     f
   }
@@ -146,8 +146,13 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
       tcl(frame2.cvs, "delete", "browse")
     tcl(frame5.cvs, "delete", "pal")
     pal.cols <- pal(n)
-    if (any(is.na(pal.cols)))
-      return()
+    pal.cols[is.na(pal.cols)] <- "#FFFFFF"
+    if (as.logical(as.integer(tclvalue(desaturation.var))))
+      pal.cols <- desaturate(pal.cols)
+    if (as.logical(as.integer(tclvalue(colorblind.var)))) {
+      type <- as.character(tclvalue(colorblind.type.var))
+      pal.cols <- dichromat(pal.cols, type=type)
+    }
     dx <- (cvs.width - 1) / n
     x2 <- 1
     y1 <- 1
@@ -158,6 +163,7 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
       pts <- .Tcl.args(c(x1, y1, x2, y1, x2, y2, x1, y2))
       tkcreate(frame5.cvs, "polygon", pts, fill=i, tag="pal")
     }
+    RegenExample(pal.cols)
   }
 
   # Update data type
@@ -184,8 +190,10 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
     x1 <- 10
     for (i in 1:length(default.pals)) {
       pal <- do.call(GetPalette, args=as.list(default.pals[[i]]))
+      pal.cols <- pal(5)
+      pal.cols[is.na(pal.cols)] <- "#FFFFFF"
       y2 <- 10
-      for (j in pal(5)) {
+      for (j in pal.cols) {
         x2 <- x1 + 20
         y1 <- y2
         y2 <- y1 + 10
@@ -306,6 +314,35 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
     tclvalue(p2.scl.var) <- p2
   }
 
+  # Show example plot
+
+  ShowExample <- function() {
+    if (!dev.example %in% dev.list()) {
+      x11(width=8, height=4)
+      dev.example <<- dev.cur()
+    }
+    par(mfrow=c(1, 2), oma=c(0, 0, 0, 0), mar=c(0, 0, 0, 0))
+    DrawPalette(is.n=TRUE)
+  }
+
+  # Regenerate example plot
+
+  RegenExample <- function(pal.cols) {
+    if (dev.example %in% dev.list())
+      dev.set(which=dev.example)
+    else
+      return()
+    n <- length(pal.cols)
+
+    # Mosaic
+    image(msc.matrix[[n]], col=pal.cols, xaxt="n", yaxt="n")
+    box()
+
+    # Filled-contour
+    image(volcano, col=rev(pal.cols), xaxt="n", yaxt="n")
+    box()
+  }
+
 
   # Main program
 
@@ -317,30 +354,66 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
 
   default.pals <- NULL
 
+  # Flag graphics device
+
+  dev.example <- 1
+
+  # Initialize example plot
+
+  msc.matrix <- list()
+  for (i in 1:50) {
+    msc.matrix[[i]] <- matrix(runif(i * 10, min=-1, max=1),
+                              nrow=10, ncol=i)
+  }
+
   # Set default and initial palettes
 
-  vars <- c("h1", "h2", "c1", "c2", "l1", "l2", "p1", "p2")
   h1 <- h2 <- c1 <- c2 <- l1 <- l2 <- p1 <- p2 <- 0
+  vars <- c("h1", "h2", "c1", "c2", "l1", "l2", "p1", "p2")
 
   qual.pals <- list()
-  qual.pals[[1]] <- c( 30,  300,  50, NA, 70, NA,  NA,  NA)
-  qual.pals[[2]] <- c( 60,  240,  50, NA, 70, NA,  NA,  NA)
-  qual.pals[[3]] <- c(270,  150,  50, NA, 70, NA,  NA,  NA)
-  qual.pals[[4]] <- c( 90,  -30,  50, NA, 70, NA,  NA,  NA)
+  qual.pals[[1]]  <- c(  0,  288,  35, NA, 85, NA,  NA,  NA) # Pastel1
+  qual.pals[[2]]  <- c( 10,  320,  50, NA, 80, NA,  NA,  NA) # Set3
+  qual.pals[[3]]  <- c(  0,  288,  60, NA, 70, NA,  NA,  NA) # Set2
+  qual.pals[[4]]  <- c(  0,  288,  50, NA, 60, NA,  NA,  NA) # Dark2
+  qual.pals[[5]]  <- c( 90,  -30,  50, NA, 70, NA,  NA,  NA)
+  qual.pals[[6]]  <- c(270,  150,  50, NA, 70, NA,  NA,  NA)
+  qual.pals[[7]]  <- c( 60,  240,  50, NA, 70, NA,  NA,  NA)
+  qual.pals[[8]]  <- c( 30,  300,  50, NA, 70, NA,  NA,  NA)
+  qual.pals[[9]]  <- c(  0,  300,  80, NA, 60, NA,  NA,  NA)
+
   seqs.pals <- list()
-  seqs.pals[[1]] <- c(260,   NA,  80,  0, 30, 90, 1.5,  NA)
-  seqs.pals[[2]] <- c(260,   NA,   0,  0, 30, 90, 1.5,  NA)
+  seqs.pals[[1]]  <- c(  0,   NA,   0,  0, 15, 95, 1.3,  NA) # Greys
+  seqs.pals[[2]]  <- c(280,  260,  60,  5, 20, 95, 0.7, 1.3) # Purples
+  seqs.pals[[3]]  <- c(260,  230,  80, 10, 30, 95, 0.7, 1.3) # Blues
+  seqs.pals[[4]]  <- c(135,  120,  50, 10, 40, 95, 0.4, 1.3) # Greens
+  seqs.pals[[5]]  <- c( 20,   45,  80,  5, 35, 95, 0.6, 1.3) # Oranges
+  seqs.pals[[6]]  <- c( 10,   40,  80, 10, 30, 95, 0.7, 1.3) # Greys
+  seqs.pals[[7]]  <- c(260,   NA,   0,  0, 30, 90, 1.5,  NA)
+  seqs.pals[[8]]  <- c(260,   NA,  80,  0, 30, 90, 1.5,  NA)
+
   seqm.pals <- list()
-  seqm.pals[[1]] <- c(  0,   90, 100, 30, 50, 90, 0.2, 1.0)
-  seqm.pals[[2]] <- c(  0,   90,  80, 30, 30, 90, 0.2, 2.0)
-  seqm.pals[[3]] <- c(130,   30,  80,  0, 60, 95, 0.1, 1.0)
-  seqm.pals[[4]] <- c(130,   30,  65,  0, 45, 90, 0.5, 1.5)
-  seqm.pals[[5]] <- c(  0, -100,  40, 80, 75, 40, 1.0, 0.0)
+  seqm.pals[[1]]  <- c(300,  200,  60,  0, 25, 95, 0.7, 1.3) # BuPu
+  seqm.pals[[2]]  <- c(370,  280,  80,  5, 25, 95, 0.7, 1.3) # PuRd
+  seqm.pals[[3]]  <- c(140,   80,  40, 10, 35, 95, 0.7, 1.7) # YlGn
+  seqm.pals[[4]]  <- c(265,   80,  60, 10, 25, 95, 0.7, 2.0) # YlGnBu
+  seqm.pals[[5]]  <- c( 10,   85,  80, 10, 25, 95, 0.4, 1.3) # YlOrRd
+  seqm.pals[[6]]  <- c(  0,   90,  80, 30, 30, 90, 0.2, 2.0)
+  seqm.pals[[7]]  <- c(  0,   90, 100, 30, 50, 90, 0.2, 1.0)
+  seqm.pals[[8]]  <- c(130,   30,  65,  0, 45, 90, 0.5, 1.5)
+  seqm.pals[[9]]  <- c(130,   30,  80,  0, 60, 95, 0.1, 1.0)
+  seqm.pals[[10]] <- c(  0, -100,  40, 80, 75, 40, 1.0, 0.0)
+
   dive.pals <- list()
-  dive.pals[[1]] <- c(260,    0,  80, NA, 30, 90, 1.5,  NA)
-  dive.pals[[2]] <- c(260,    0, 100, NA, 50, 90, 1.0,  NA)
-  dive.pals[[3]] <- c(130,   43, 100, NA, 70, 90, 1.0,  NA)
-  dive.pals[[4]] <- c(180,  330,  59, NA, 75, 95, 1.5,  NA)
+  dive.pals[[1]]  <- c(340,  128,  45, NA, 35, 95, 0.7, 1.3) # PiYG
+  dive.pals[[2]]  <- c(300,  128,  45, NA, 30, 95, 0.7, 1.3) # PRGn
+  dive.pals[[3]]  <- c( 55,  160,  30, NA, 35, 95, 0.7, 1.3) # BrBG
+  dive.pals[[4]]  <- c( 40,  270,  45, NA, 30, 95, 0.7, 1.3) # PuOr
+  dive.pals[[5]]  <- c( 12,  265,  80, NA, 25, 95, 0.7, 1.3) # RdBu
+  dive.pals[[6]]  <- c(260,    0,  80, NA, 30, 90, 1.5,  NA)
+  dive.pals[[7]]  <- c(260,    0, 100, NA, 50, 90, 1.0,  NA)
+  dive.pals[[8]]  <- c(130,   43, 100, NA, 70, 90, 1.0,  NA)
+  dive.pals[[9]]  <- c(180,  330,  59, NA, 75, 95, 1.5,  NA)
 
   # Set limits for palette attributes
 
@@ -348,11 +421,11 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
   h.lim <- c(-360, 360)
   c.lim <- c(   0, 100)
   l.lim <- c(   0, 100)
-  p.lim <- c(   0,   5)
+  p.lim <- c(   0,   2)
 
   # Set dimensions on palette canvas
 
-  cvs.width <- 350
+  cvs.width <- 328 # 30 * 10 + 10 + 18
   cvs.height <- 25
 
   # Assign additional variables linked to Tk widgets
@@ -379,6 +452,10 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
   p2.scl.var <- tclVar()
   p2.ent.var <- tclVar()
 
+  desaturation.var <- tclVar(FALSE)
+  colorblind.var <- tclVar(FALSE)
+  colorblind.type.var <- tclVar("deutan")
+
   tt.done.var <- tclVar(0)
 
   # Open GUI
@@ -421,23 +498,25 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
 
   tkconfigure(tt, menu=top.menu)
 
-  # Frame 0, ok and cancel buttons
+  # Frame 0, example, ok, and cancel buttons
 
   frame0 <- ttkframe(tt, relief="flat")
 
-  frame0.but.1 <- ttkbutton(frame0, width=12, text="OK", command=SavePalette)
-  frame0.but.2 <- ttkbutton(frame0, width=12, text="Cancel",
+  frame0.but.1 <- ttkbutton(frame0, width=12, text="Example",
+                            command=ShowExample)
+  frame0.but.3 <- ttkbutton(frame0, width=12, text="OK", command=SavePalette)
+  frame0.but.4 <- ttkbutton(frame0, width=12, text="Cancel",
                             command=function() {
                               pal.rtn <<- NULL
                               tclvalue(tt.done.var) <- 1
                             })
+  tkgrid(frame0.but.1, "x", frame0.but.3, frame0.but.4, pady=c(10, 10))
+  tkgrid.configure(frame0.but.1, sticky="w", padx=c(10, 0))
+  tkgrid.configure(frame0.but.3, sticky="e")
+  tkgrid.configure(frame0.but.4, sticky="w", padx=c(4, 10))
+  tkgrid.columnconfigure(frame0, 1, weight=1)
 
-  tkgrid(frame0.but.1, frame0.but.2, pady=c(10, 10))
-
-  tkgrid.configure(frame0.but.1, sticky="e")
-  tkgrid.configure(frame0.but.2, sticky="w", padx=c(4, 10))
-
-  tkpack(frame0, side="bottom", anchor="e")
+  tkpack(frame0, fill="x", side="bottom", anchor="e")
 
   # Frame 1, choose nature of data
 
@@ -448,7 +527,7 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
                                        "Sequential (multiple hues)",
                                        "Diverging"))
 
-  tkgrid(frame1.lab.1, frame1.box.2, pady=10)
+  tkgrid(frame1.lab.1, frame1.box.2, pady=c(10, 0))
   tkgrid.configure(frame1.lab.1, padx=c(10, 2))
   tkgrid.configure(frame1.box.2, padx=c(0, 10), sticky="we")
 
@@ -460,16 +539,16 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
 
   frame2 <- ttklabelframe(tt, relief="flat", borderwidth=5, padding=5,
                           text="Default color schemes")
-  frame2.cvs <- tkcanvas(frame2, relief="flat", width=50, height=70,
+  frame2.cvs <- tkcanvas(frame2, relief="flat", width=30 * 10 + 10, height=70,
                          background="white", confine=TRUE, closeenough=0,
                          borderwidth=0, highlightthickness=0)
   tkgrid(frame2.cvs, sticky="we")
   tkgrid.columnconfigure(frame2, 0, weight=1)
-  tkpack(frame2, fill="x", padx=10)
+  tkpack(frame2, fill="x", padx=10, pady=10)
 
   # Frame 3, color description
 
-  txt <- "Color description: Hue, Croma, Luminance, Power"
+  txt <- "Color description: Hue, Chroma, Luminance, Power"
   frame3 <- ttklabelframe(tt, relief="flat", borderwidth=5, padding=5, text=txt)
 
   frame3.lab.1.1 <- ttklabel(frame3, text="H1", width=2)
@@ -555,7 +634,7 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
 
   tkgrid.columnconfigure(frame3, 1, weight=1)
 
-  tkpack(frame3, fill="x", padx=10, pady=10)
+  tkpack(frame3, fill="x", padx=10, pady=0)
 
   # Frame 4, number of colors in palette
 
@@ -575,17 +654,45 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
   tkgrid.configure(frame4.scl.2, sticky="we", padx=c(4, 10))
   tkgrid.columnconfigure(frame4, 1, weight=1)
 
-  tkpack(frame4, fill="x", padx=10)
+  tkpack(frame4, fill="x", padx=10, pady=10)
 
- # Frame 5, color palette
+ # Frame 5, color palette and robustness checks
 
   frame5 <- ttkframe(tt, relief="flat")
   frame5.cvs <- tkcanvas(frame5, relief="flat",
                          width=cvs.width + 1, height=cvs.height + 1,
                          background="black", confine=TRUE, closeenough=0,
                          borderwidth=0, highlightthickness=0)
-  tkgrid(frame5.cvs, padx=10, pady=c(10, 0))
-  tkpack(frame5)
+  tkgrid(frame5.cvs, padx=10, pady=0)
+
+  frame5.chk.1 <- ttkcheckbutton(frame5, text="Desaturation",
+                                 variable=desaturation.var,
+                                 command=function() DrawPalette(is.n=TRUE))
+
+  is.pkg <- "dichromat" %in% .packages(all.available=TRUE) &&
+            require(dichromat, quietly=FALSE)
+  if (is.pkg) {
+    frame5.chk.2 <- ttkcheckbutton(frame5, text="Color blindness:",
+                                   variable=colorblind.var,
+                                   command=function() DrawPalette(is.n=TRUE))
+    frame5.rb.3 <- ttkradiobutton(frame5, variable=colorblind.type.var,
+                                  value="deutan", text="deutan",
+                                  command=function() DrawPalette(is.n=TRUE))
+    frame5.rb.4 <- ttkradiobutton(frame5, variable=colorblind.type.var,
+                                  value="protan", text="protan",
+                                  command=function() DrawPalette(is.n=TRUE))
+    tkgrid(frame5.chk.1, frame5.chk.2, frame5.rb.3, frame5.rb.4, "x",
+           pady=c(2, 0), sticky="w")
+    tkgrid.configure(frame5.chk.2, padx=c(7, 0))
+    tkgrid.configure(frame5.cvs, columnspan=5)
+    tkgrid.columnconfigure(frame5, 4, weight=1)
+  } else {
+    tkgrid(frame5.chk.1, "x", pady=c(2, 0), sticky="w")
+    tkgrid.configure(frame5.cvs, columnspan=2)
+    tkgrid.columnconfigure(frame5, 1, weight=1)
+  }
+  tkgrid.configure(frame5.chk.1, padx=c(10, 0))
+  tkpack(frame5, fill="x")
 
   # Initial commands
 
@@ -637,5 +744,8 @@ ChoosePalette <- function(pal=terrain_hcl, n=7L, parent=NULL) {
   tkdestroy(tt)
   tclServiceMode(TRUE)
 
+  if (dev.example %in% dev.list())
+    dev.off(which=dev.example)
+
   invisible(pal.rtn)
 }
